#include <windows.h>
#include "Intlib.h"
#include "GpioLib.h"

#define GPIO -1
#define INTERRUPT_PIN	101
#define OUTPUT_PIN		135
#define TIMEOUT			10000							///< Time-out in ms.
 
int wmain()
{
    PIN_INSTANCE gpio;
    HANDLE hEvent = NULL;  
    BOOL pinLevel = FALSE;
    BOOL returnFlag = FALSE;
	DWORD retVal = 0;
    DWORD irq = 0;   
    DWORD gpioNumber = 0;
    DWORD interruptEdge = 0;
    DWORD systemInterrupt = 0;
	CHAR select = 0;

    SetPinAltFn(INTERRUPT_PIN, (DWORD)GPIO, DIR_IN);   ///< Sets INTERRUPT_PIN as GPIO input to capture the interrupt.
    SetPinAltFn(OUTPUT_PIN, (DWORD)GPIO, DIR_OUT);     ///< Sets OUTPUT_PIN as GPIO output to blink the LED.
	SetPinLevel(OUTPUT_PIN, FALSE);
    GetGPIOFromPin(INTERRUPT_PIN, FALSE, &gpio);       ///< Gets GPIO number for INTERRUPT_PIN.
    gpioNumber = gpio.inst1;
    printf("\nGPIO number for SODIMM pin %d is: %d.", INTERRUPT_PIN, gpioNumber);
    irq = GetGPIOIrq(gpioNumber);                      ///< Requests interrupt (IRQ) on GPIO number.
    if (!irq)
    {
        printf("\nCannot obtain IRQ for GPIO Number %d. \nPress enter to exit.", gpioNumber);
        getchar();
        return FALSE;
    }
	printf("\nIRQ number for GPIO %d is: %d", gpioNumber, irq);
    interruptEdge = GPIO_EDGE_RISING;                   ///< Configures external interrupt signal on rising edge.
    if (!SetGPIOIrqEdge(gpioNumber, interruptEdge)) 
    {
        printf("\nCannot set GPIO interrupt edge detect.");
        getchar();
        return FALSE;
    }
    hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);     ///< Creates event or check for existing event to wait for.
    if (!hEvent) 
    {
        printf("\nEvent cannot be created.");
        getchar();
        return FALSE;
    }
    systemInterrupt = RequestSysInterrupt(irq);         ///< Gets the system interrupt number for corresponding irq.
    if (!systemInterrupt)
    {
        printf("\nCannot obtain system interrupt number.");
        getchar();
        return FALSE;
    }
    printf("\nSystem Interrupt request: %d", systemInterrupt);
 
    if (!InterruptInitializeCompat(systemInterrupt, hEvent, NULL, 0))     ///< Initializes system interrupt.
    {
        printf("\nCannot Initialize interrupt.");
        ReleaseSysIntr(systemInterrupt);
        getchar();
        return FALSE;
    }
    CeSetThreadPriority(GetCurrentThread(), 200);
    while (TRUE)
    {
        /// Waits for Event (Interrupt).
		retVal = WaitForSingleObject(hEvent, TIMEOUT);
        if (retVal == WAIT_OBJECT_0)
        {
            if (pinLevel == TRUE)
            {
                returnFlag = SetPinLevel(OUTPUT_PIN, FALSE);
				pinLevel = FALSE;
            }
            else 
            {
                returnFlag = SetPinLevel(OUTPUT_PIN, TRUE);
				pinLevel = TRUE;
            }
            if (MessageBox(NULL, L"Interrupt Event Detected, continue Waiting?", L"Interrupt", MB_YESNO) == IDNO)
			{
				break;
			}
            InterruptDoneCompat(systemInterrupt);
        }
		else if (retVal == WAIT_TIMEOUT)
		{
			printf("\nTimeout %d ms, No interrupt captured on SODIMM pin %d", TIMEOUT, INTERRUPT_PIN);
			printf("\nContinue waiting - y/n?  ");
			fflush(stdin);
			scanf_s("%c", &select);
			if (select == 'y' || select == 'Y')
			{
				continue;
			}
			else
			{
				break;
			}
		}
		else
		{
			printf("\nWait failure. Error Code = %d", GetLastError());
			break;
		}
    }
    InterruptDisableCompat(systemInterrupt);			///< Disables interrupt.
	CloseHandle(hEvent);								///< Closes the event handle.
    if (!ReleaseSysIntr(systemInterrupt)) 
    {
        MessageBox(NULL, L"An Error occurred!\nInterrupt was not released correctly", L"ERROR", MB_OK);
        return FALSE;
    }
    return TRUE;
}